import React, { useState, useCallback, useEffect } from 'react';
import {
  StyleButtons,
  type StyleType,
} from './components/appSections/StyleButtons';
import {
  CaseButtons,
  type CaseType,
} from './components/appSections/CaseButtons';
import { TextInput } from './components/appSections/TextInput';
import { ConvertedOutput } from './components/appSections/ConvertedOutput';
import { PreserveWordsModal } from './components/PreserveWordsModal';
import {
  convertToSentenceCase,
  convertToCamelCase,
  convertToSnakeCase,
  convertToKebabCase,
  convertToTitleCase,
  convertToUpperCase,
  convertToLowerCase,
  convertToFirstLetter,
  convertToAltCase,
  convertToToggleCase,
} from './utils/caseConverters';
import {
  findNonUnicodeCharacters,
  convertNonUnicodeToUnicode,
} from './utils/textUtils';

function App() {
  const [text, setText] = useState('');
  const [selectedStyle, setSelectedStyle] = useState<StyleType>('AP');
  const [selectedCase, setSelectedCase] = useState<CaseType>('sentence');
  const [isUploadMode, setIsUploadMode] = useState(false);
  const [isPreserveModalOpen, setIsPreserveModalOpen] = useState(false);
  const [preservedWords, setPreservedWords] = useState('');
  const [nonUnicodeChars, setNonUnicodeChars] = useState<string[]>([]);

  // Load preserved words from localStorage when the app initializes
  useEffect(() => {
    const savedWords = localStorage.getItem('preservedWords');
    if (savedWords !== null) {
      setPreservedWords(savedWords);
    }
  }, []);

  const handleFileUpload = useCallback((file: File) => {
    const reader = new FileReader();
    reader.onload = (e) => {
      const content = e.target?.result as string;
      setText(content);
      setNonUnicodeChars(findNonUnicodeCharacters(content));
    };
    reader.readAsText(file);
  }, []);

  const handleTextChange = useCallback((newText: string) => {
    setText(newText);
    setNonUnicodeChars(findNonUnicodeCharacters(newText));
  }, []);

  const convertText = useCallback(
    (input: string): string => {
      if (!input) return '';

      const normalizedText = convertNonUnicodeToUnicode(input);
      const preservedWordsList = preservedWords
        .split('\n')
        .map((word) => word.trim())
        .filter(Boolean);

      let convertedText = '';
      switch (selectedCase) {
        case 'sentence':
          convertedText = convertToSentenceCase(
            normalizedText,
            preservedWordsList
          );
          break;
        case 'title':
          convertedText = convertToTitleCase(
            normalizedText,
            preservedWordsList,
            selectedStyle
          );
          break;
        case 'capitalize':
          convertedText = convertToFirstLetter(
            normalizedText,
            preservedWordsList
          );
          break;
        case 'uppercase':
          convertedText = convertToUpperCase(
            normalizedText,
            preservedWordsList
          );
          break;
        case 'lowercase':
          convertedText = convertToLowerCase(
            normalizedText,
            preservedWordsList
          );
          break;
        case 'firstLetter':
          convertedText = convertToFirstLetter(
            normalizedText,
            preservedWordsList
          );
          break;
        case 'alt':
          convertedText = convertToAltCase(normalizedText, preservedWordsList);
          break;
        case 'toggle':
          convertedText = convertToToggleCase(
            normalizedText,
            preservedWordsList
          );
          break;
        default:
          convertedText = normalizedText;
      }

      return convertedText;
    },
    [selectedCase, selectedStyle, preservedWords]
  );

  const convertedText = convertText(text);

  const copyToClipboard = async () => {
    try {
      await navigator.clipboard.writeText(convertedText);
    } catch (err) {
      console.error('Failed to copy text:', err);
    }
  };

  return (
    <div className="p-6 text-sm font-bold text-gray-900">
      <div className="max-w-4xl mx-auto space-y-6">
        <div>
          <div className="flex justify-between items-center mb-6">
            <StyleButtons
              selectedStyle={selectedStyle}
              onStyleSelect={setSelectedStyle}
            />
            <CaseButtons
              selectedCase={selectedCase}
              onCaseSelect={setSelectedCase}
            />
          </div>

          <TextInput
            text={text}
            onTextChange={handleTextChange}
            isUploadMode={isUploadMode}
            onUploadModeToggle={() => setIsUploadMode(!isUploadMode)}
            onPreserveClick={() => setIsPreserveModalOpen(true)}
            onFileUpload={handleFileUpload}
            lockedWordsCount={preservedWords
              .split('\n')
              .filter((line) => line.trim().length > 0).length}
            isLocked={preservedWords.trim().length > 0}
          />
        </div>

        <ConvertedOutput
          text={convertedText}
          onCopy={copyToClipboard}
          nonUnicodeChars={nonUnicodeChars}
        />
        
        {/* FAQ Section */}
        {/* <FAQ /> */}
      </div>

      <PreserveWordsModal
        isOpen={isPreserveModalOpen}
        onClose={() => setIsPreserveModalOpen(false)}
        preservedWords={preservedWords}
        onPreservedWordsChange={setPreservedWords}
      />
    </div>
  );
}

export default App;
